@@ -21,27 +21,29 @@ from mama_cas.exceptions import InternalError
 from mama_cas.exceptions import BadPGTError
 
 
-LOG = logging.getLogger('mama_cas')
+logger = logging.getLogger(__name__)
 
 
-class NeverCache(object):
+class NeverCacheMixin(object):
     """
     View mixin for disabling caching.
     """
     @method_decorator(never_cache)
     def dispatch(self, request, *args, **kwargs):
-        return super(NeverCache, self).dispatch(request, *args, **kwargs)
+        return super(NeverCacheMixin, self).dispatch(request, *args, **kwargs)
 
-class LoginRequired(object):
+
+class LoginRequiredMixin(object):
     """
     View mixin to require a logged in user.
     """
     @method_decorator(login_required(login_url=reverse_lazy('cas_login'),
                                      redirect_field_name=None))
     def dispatch(self, request, *args, **kwargs):
-        return super(LoginRequired, self).dispatch(request, *args, **kwargs)
+        return super(LoginRequiredMixin, self).dispatch(request, *args, **kwargs)
 
-class ValidateTicket(object):
+
+class ValidateTicketMixin(object):
     """
     View mixin providing ticket validation methods.
     """
@@ -58,18 +60,19 @@ class ValidateTicket(object):
         renew = request.GET.get('renew')
         pgturl = request.GET.get('pgtUrl')
 
-        LOG.debug("Service validation request received for %s" % ticket)
+        logger.debug("Service validation request received for %s" % ticket)
         try:
             st = ServiceTicket.objects.validate_ticket(ticket,
                                                        service=service,
                                                        renew=renew)
         except (InvalidRequestError, InvalidTicketError,
                 InvalidServiceError, InternalError) as e:
-            LOG.warn("%s %s" % (e.code, e))
+            logger.warn("%s %s" % (e.code, e))
             return None, None, e
         else:
             if pgturl:
-                LOG.debug("Proxy-granting ticket request received for %s" % pgturl)
+                logger.debug("Proxy-granting ticket request received for %s" %
+                             pgturl)
                 pgt = ProxyGrantingTicket.objects.create_ticket(pgturl,
                                                                 user=st.user,
                                                                 granted_by_st=st)
@@ -90,13 +93,13 @@ class ValidateTicket(object):
         ticket = request.GET.get('ticket')
         pgturl = request.GET.get('pgtUrl')
 
-        LOG.debug("Proxy validation request received for %s" % ticket)
+        logger.debug("Proxy validation request received for %s" % ticket)
         try:
             pt = ProxyTicket.objects.validate_ticket(ticket,
                                                      service=service)
         except (InvalidRequestError, InvalidTicketError,
                 InvalidServiceError, InternalError) as e:
-            LOG.warn("%s %s" % (e.code, e))
+            logger.warn("%s %s" % (e.code, e))
             return None, None, None, e
         else:
             # Build a list of all services that proxied authentication,
@@ -108,7 +111,8 @@ class ValidateTicket(object):
                 prior_pt = prior_pt.granted_by_pgt.granted_by_pt
 
             if pgturl:
-                LOG.debug("Proxy-granting ticket request received for %s" % pgturl)
+                logger.debug("Proxy-granting ticket request received for %s" %
+                             pgturl)
                 pgt = ProxyGrantingTicket.objects.create_ticket(pgturl,
                                                                 user=pt.user,
                                                                 granted_by_pt=pt)
@@ -126,12 +130,14 @@ class ValidateTicket(object):
         pgt = request.GET.get('pgt')
         target_service = request.GET.get('targetService')
 
-        LOG.debug("Proxy ticket request received")
+        logger.debug("Proxy ticket request received for %s using %s" %
+                     (target_service, pgt))
         try:
             pgt = ProxyGrantingTicket.objects.validate_ticket(pgt,
                                                               target_service)
-        except (InvalidRequestError, BadPGTError, InternalError) as e:
-            LOG.warn("%s %s" % (e.code, e))
+        except (InvalidRequestError, BadPGTError,
+                InvalidServiceError, InternalError) as e:
+            logger.warn("%s %s" % (e.code, e))
             return None, e
         else:
             pt = ProxyTicket.objects.create_ticket(service=target_service,
@@ -139,7 +145,8 @@ class ValidateTicket(object):
                                                    granted_by_pgt=pgt)
             return pt, None
 
-class CustomAttributes(object):
+
+class CustomAttributesMixin(object):
     """
     View mixin for including custom user attributes in a validation response.
     """
@@ -172,7 +179,7 @@ class CustomAttributes(object):
             try:
                 attribute = [name, getattr(user, attr)]
             except AttributeError:
-                LOG.warn("User has no attribute named '%s'" % attr)
+                logger.error("User has no attribute named '%s'" % attr)
             else:
                 attributes.append(attribute)
 
@@ -186,13 +193,14 @@ class CustomAttributes(object):
                 try:
                     attribute = [name, getattr(profile, attr)]
                 except AttributeError:
-                    LOG.warn("Profile has no attribute named '%s'" % attr)
+                    logger.error("Profile has no attribute named '%s'" % attr)
                 else:
                     attributes.append(attribute)
 
         return attributes
 
-class LogoutUser(object):
+
+class LogoutUserMixin(object):
     """
     View mixin for logging a user out of a single sign-on session.
     """
@@ -208,10 +216,11 @@ class LogoutUser(object):
            existing session data for the current request.
         """
         if request.user.is_authenticated():
-            # Ensure all tickets created for this user are consumed
             ServiceTicket.objects.consume_tickets(request.user)
             ProxyTicket.objects.consume_tickets(request.user)
             ProxyGrantingTicket.objects.consume_tickets(request.user)
 
+            logger.info("Single sign-on session ended for %s" % request.user)
             logout(request)
-            messages.success(request, _("You have been successfully logged out"))
+            messages.success(request,
+                             _("You have been successfully logged out"))
