@@ -6,6 +6,7 @@ from django.test import TestCase
 from django.utils.timezone import now
 from django.core import management
 from django.contrib.auth.models import User
+from django.conf import settings
 
 from mama_cas.models import ServiceTicket
 from mama_cas.models import ProxyTicket
@@ -27,10 +28,10 @@ class ServiceTicketTests(TestCase):
     valid_st_regexp = '^ST-[0-9]{10,}-[a-zA-Z0-9]{32}$'
     valid_service = 'http://www.example.com/'
     invalid_service = 'http://www.example.org/'
-    user_info = { 'username': 'ellen',
-                  'password': 'mamas&papas',
-                  'email': 'ellen@example.com' }
-    ticket_info = { 'service': valid_service }
+    user_info = {'username': 'ellen',
+                 'password': 'mamas&papas',
+                 'email': 'ellen@example.com'}
+    ticket_info = {'service': valid_service}
 
     def setUp(self):
         """
@@ -40,6 +41,15 @@ class ServiceTicketTests(TestCase):
         self.user = User.objects.create_user(**self.user_info)
         self.ticket_info.update({'user': self.user})
 
+        self.old_valid_services = getattr(settings, 'MAMA_CAS_VALID_SERVICES', ())
+        settings.MAMA_CAS_VALID_SERVICES = (self.valid_service,)
+
+    def tearDown(self):
+        """
+        Undo any modifications made to settings.
+        """
+        settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
+
     def test_create_ticket(self):
         """
         A new ``ServiceTicket`` ought to exist in the database with a valid
@@ -75,10 +85,13 @@ class ServiceTicketTests(TestCase):
     def test_validate_ticket_no_service(self):
         """
         The ``ServiceTicket`` validation process ought to fail when no service
-        identifier is provided.
+        identifier is provided and the ticket ought to be consumed.
         """
-        self.assertRaises(InvalidRequestError, ServiceTicket.objects.validate_ticket,
-                          self.valid_st_str, service=None)
+        st = ServiceTicket.objects.create_ticket(**self.ticket_info)
+        self.assertRaises(InvalidRequestError,
+                          ServiceTicket.objects.validate_ticket,
+                          st.ticket, service=None)
+        self.assertTrue(ServiceTicket.objects.get(ticket=st.ticket).is_consumed())
 
     def test_validate_ticket_invalid_ticket(self):
         """
@@ -134,15 +147,17 @@ class ServiceTicketTests(TestCase):
         the presentation of the user's primary credentials.
         """
         st = ServiceTicket.objects.create_ticket(**self.ticket_info)
-        self.assertRaises(InvalidTicketError, ServiceTicket.objects.validate_ticket,
+        self.assertRaises(InvalidTicketError,
+                          ServiceTicket.objects.validate_ticket,
                           st.ticket,
                           service=self.valid_service,
                           renew=True)
 
-        st = ServiceTicket.objects.create_ticket(primary=True, **self.ticket_info)
-        self.assertTrue(ServiceTicket.objects.validate_ticket(st.ticket,
-                                                              service=st.service,
-                                                              renew=True), st)
+        st = ServiceTicket.objects.create_ticket(primary=True,
+                                                 **self.ticket_info)
+        self.assertEqual(ServiceTicket.objects.validate_ticket(st.ticket,
+                                                               service=self.valid_service,
+                                                               renew=True), st)
 
     def test_invalid_ticket_deletion(self):
         """
@@ -194,6 +209,7 @@ class ServiceTicketTests(TestCase):
         self.assertRaises(ServiceTicket.DoesNotExist, ServiceTicket.objects.get,
                           ticket=consumed_st.ticket)
 
+
 class ProxyTicketTests(TestCase):
     """
     Test the model and manager used for ``ProxyTicket``s.
@@ -202,10 +218,10 @@ class ProxyTicketTests(TestCase):
     valid_pt_regexp = '^PT-[0-9]{10,}-[a-zA-Z0-9]{32}$'
     valid_service = 'http://www.example.com/'
     invalid_service = 'http://www.example.org/'
-    user_info = { 'username': 'ellen',
-                  'password': 'mamas&papas',
-                  'email': 'ellen@example.com' }
-    ticket_info = { 'service': valid_service }
+    user_info = {'username': 'ellen',
+                 'password': 'mamas&papas',
+                 'email': 'ellen@example.com'}
+    ticket_info = {'service': valid_service}
 
     def setUp(self):
         """
@@ -217,7 +233,16 @@ class ProxyTicketTests(TestCase):
         self.pgt = ProxyGrantingTicket.objects.create_ticket(self.valid_service,
                                                              validate=False,
                                                              user=self.user)
-        self.ticket_info.update({ 'user': self.user, 'granted_by_pgt': self.pgt })
+        self.ticket_info.update({'user': self.user, 'granted_by_pgt': self.pgt})
+
+        self.old_valid_services = getattr(settings, 'MAMA_CAS_VALID_SERVICES', ())
+        settings.MAMA_CAS_VALID_SERVICES = (self.valid_service,)
+
+    def tearDown(self):
+        """
+        Undo any modifications made to settings.
+        """
+        settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
 
     def test_create_ticket(self):
         """
@@ -255,10 +280,13 @@ class ProxyTicketTests(TestCase):
     def test_validate_ticket_no_service(self):
         """
         The ``ProxyTicket`` validation process ought to fail when no service
-        identifier is provided.
+        identifier is provided and the ticket ought to be consumed.
         """
-        self.assertRaises(InvalidRequestError, ProxyTicket.objects.validate_ticket,
-                          self.valid_pt_str, service=None)
+        pt = ProxyTicket.objects.create_ticket(**self.ticket_info)
+        self.assertRaises(InvalidRequestError,
+                          ProxyTicket.objects.validate_ticket,
+                          pt.ticket, service=None)
+        self.assertTrue(ProxyTicket.objects.get(ticket=pt.ticket).is_consumed())
 
     def test_validate_ticket_invalid_ticket(self):
         """
@@ -357,6 +385,7 @@ class ProxyTicketTests(TestCase):
         self.assertRaises(ProxyTicket.DoesNotExist, ProxyTicket.objects.get,
                           ticket=consumed_pt.ticket)
 
+
 class ProxyGrantingTicketTests(TestCase):
     """
     Test the model and manager used for ``ProxyGrantingTicket``s.
@@ -366,9 +395,9 @@ class ProxyGrantingTicketTests(TestCase):
     valid_pgtiou_str = 'PGTIOU-0000000000-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
     valid_pgtiou_regexp = '^PGTIOU-[0-9]{10,}-[a-zA-Z0-9]{32}$'
     valid_service = 'http://www.example.com/'
-    user_info = { 'username': 'ellen',
-                  'password': 'mamas&papas',
-                  'email': 'ellen@example.com' }
+    user_info = {'username': 'ellen',
+                 'password': 'mamas&papas',
+                 'email': 'ellen@example.com'}
     ticket_info = {}
 
     def setUp(self):
@@ -379,6 +408,15 @@ class ProxyGrantingTicketTests(TestCase):
         self.user = User.objects.create_user(**self.user_info)
         self.ticket_info.update({'user': self.user})
 
+        self.old_valid_services = getattr(settings, 'MAMA_CAS_VALID_SERVICES', ())
+        settings.MAMA_CAS_VALID_SERVICES = (self.valid_service,)
+
+    def tearDown(self):
+        """
+        Undo any modifications made to settings.
+        """
+        settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
+
     def test_create_ticket(self):
         """
         A new ``ProxyGrantingTicket`` ought to exist in the database with a
@@ -386,8 +424,8 @@ class ProxyGrantingTicketTests(TestCase):
         the ``User`` that authorized its creation.
         """
         pgt = ProxyGrantingTicket.objects.create_ticket(self.valid_service,
-                                                       validate=False,
-                                                       **self.ticket_info)
+                                                        validate=False,
+                                                        **self.ticket_info)
 
         self.assertEqual(ProxyGrantingTicket.objects.count(), 1)
         self.assertTrue(re.search(self.valid_pgt_regexp, pgt.ticket))
@@ -442,19 +480,6 @@ class ProxyGrantingTicketTests(TestCase):
         self.assertRaises(BadPGTError, ProxyGrantingTicket.objects.validate_ticket,
                           self.valid_pgt_str, service=self.valid_service)
 
-    def test_validate_ticket_expired_ticket(self):
-        """
-        The ``ProxyGrantingTicket`` validation process ought to fail when an expired
-        ticket is provided.
-        """
-        pgt = ProxyGrantingTicket.objects.create_ticket(self.valid_service,
-                                                       validate=False,
-                                                       **self.ticket_info)
-        pgt.created = now() - timedelta(minutes=pgt.TICKET_EXPIRE + 1)
-        pgt.save()
-        self.assertRaises(InvalidTicketError, ProxyGrantingTicket.objects.validate_ticket,
-                          pgt.ticket, service=self.valid_service)
-
     def test_validate_ticket_consumed_ticket(self):
         """
         The ``ProxyGrantingTicket`` validation process ought to fail when a consumed
