@@ -1,4 +1,6 @@
 import logging
+import random
+import string
 import urlparse
 import urllib
 from xml.etree.ElementTree import ElementTree
@@ -8,11 +10,13 @@ from django.conf import settings
 from django.test import TestCase
 from django.core.urlresolvers import reverse
 from django.contrib.auth.models import User
+from django.core.exceptions import ImproperlyConfigured
 
 from mama_cas.models import ServiceTicket
 from mama_cas.models import ProxyTicket
 from mama_cas.models import ProxyGrantingTicket
 from mama_cas.forms import LoginForm
+from mama_cas.forms import LoginFormWarn
 
 
 logging.disable(logging.CRITICAL)
@@ -24,10 +28,10 @@ class LoginViewTests(TestCase):
     """
     Test the ``LoginView`` view.
     """
-    user_info = { 'username': 'ellen',
-                  'password': 'mamas&papas',
-                  'email': 'ellen@example.com' }
-    service = { 'service': 'http://www.example.com/' }
+    user_info = {'username': 'ellen',
+                 'password': 'mamas&papas',
+                 'email': 'ellen@example.com'}
+    service = {'service': 'http://www.example.com/'}
 
     def setUp(self):
         """
@@ -80,16 +84,17 @@ class LoginViewTests(TestCase):
         self.assertEqual(destination, self.user_info['service'])
         self.assertTrue('ticket' in query)
 
+
 class WarnViewTests(TestCase):
     """
     Test the ``WarnView`` view.
     """
-    user_info = { 'username': 'ellen',
-                  'password': 'mamas&papas',
-                  'email': 'ellen@example.com' }
+    user_info = {'username': 'ellen',
+                 'password': 'mamas&papas',
+                 'email': 'ellen@example.com'}
     valid_service = 'http://www.example.com/'
     form_data = user_info.copy()
-    form_data.update({ 'warn': 'true' })
+    form_data.update({'warn': 'true'})
 
     def setUp(self):
         """
@@ -114,19 +119,17 @@ class WarnViewTests(TestCase):
         When a user logs in with the warn parameter present, the user's
         session should contain a ``warn`` attribute and a ``ServiceTicket``
         request to the credential requestor should redirect to the warn view.
-
-        NOTE: this test will fail if ``LoginView`` is not using the
-        ``LoginFormWarn`` form class.
         """
-        response = self.client.post(reverse('cas_login'), self.form_data)
+        # Only continue the test if the required form class is in use
+        response = self.client.get(reverse('cas_login'))
+        if isinstance(response.context['form'], LoginFormWarn):
+            response = self.client.post(reverse('cas_login'), self.form_data)
+            query_str = "?service=%s" % urllib.quote(self.valid_service, '')
+            response = self.client.get(reverse('cas_login') + query_str)
 
-        self.assertEqual(self.client.session.get('warn'), True)
-
-        query_str = "?service=%s" % urllib.quote(self.valid_service, '')
-        response = self.client.get(reverse('cas_login') + query_str)
-
-        self.assertRedirects(response, reverse('cas_warn') + query_str,
-                             status_code=302, target_status_code=200)
+            self.assertEqual(self.client.session.get('warn'), True)
+            self.assertRedirects(response, reverse('cas_warn') + query_str,
+                                 status_code=302, target_status_code=200)
 
     def test_warn_view_display(self):
         """
@@ -154,13 +157,14 @@ class WarnViewTests(TestCase):
         self.assertRedirects(response, reverse('cas_login') + '?warned=true',
                              status_code=302, target_status_code=200)
 
+
 class LogoutViewTests(TestCase):
     """
     Test the ``LogoutView`` view.
     """
-    user_info = { 'username': 'ellen',
-                  'password': 'mamas&papas',
-                  'email': 'ellen@example.com' }
+    user_info = {'username': 'ellen',
+                 'password': 'mamas&papas',
+                 'email': 'ellen@example.com'}
 
     def setUp(self):
         """
@@ -201,6 +205,7 @@ class LogoutViewTests(TestCase):
                              status_code=302, target_status_code=200)
         self.assertFalse('_auth_user_id' in self.client.session)
 
+
 class ValidateViewTests(TestCase):
     """
     Test the ``ValidateView`` view.
@@ -208,10 +213,10 @@ class ValidateViewTests(TestCase):
     invalid_st_str = 'ST-0000000000-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
     valid_service = 'http://www.example.com/'
     invalid_service = 'http://www.example.org/'
-    user_info = { 'username': 'ellen',
-                  'password': 'mamas&papas',
-                  'email': 'ellen@example.com' }
-    ticket_info = { 'service': valid_service }
+    user_info = {'username': 'ellen',
+                 'password': 'mamas&papas',
+                 'email': 'ellen@example.com'}
+    ticket_info = {'service': valid_service}
     validation_failure = "no\n\n"
     validation_success = "yes\n%s\n" % user_info['username']
 
@@ -220,9 +225,18 @@ class ValidateViewTests(TestCase):
         Create a valid user and service ticket for testing purposes.
         """
         self.user = User.objects.create_user(**self.user_info)
-        self.ticket_info.update({ 'user': self.user })
+        self.ticket_info.update({'user': self.user})
         self.st = ServiceTicket.objects.create_ticket(**self.ticket_info)
 
+        self.old_valid_services = getattr(settings, 'MAMA_CAS_VALID_SERVICES', ())
+        settings.MAMA_CAS_VALID_SERVICES = (self.valid_service,)
+
+    def tearDown(self):
+        """
+        Undo any modifications made to settings.
+        """
+        settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
+
     def test_validate_view(self):
         """
         When called with no parameters, a ``GET`` request to the view should
@@ -288,6 +302,7 @@ class ValidateViewTests(TestCase):
         self.assertEqual(response.content, self.validation_failure)
         self.assertEqual(response.get('Content-Type'), 'text/plain')
 
+
 class ServiceValidateViewTests(TestCase):
     """
     Test the ``ServiceValidateView`` view.
@@ -295,15 +310,17 @@ class ServiceValidateViewTests(TestCase):
     invalid_st_str = 'ST-0000000000-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
     valid_service = 'http://www.example.com/'
     invalid_service = 'http://www.example.org/'
-    user_info = { 'username': 'ellen',
-                  'password': 'mamas&papas',
-                  'email': 'ellen@example.com' }
-    user_attr = { 'givenName': 'first_name',
-                  'sn': 'last_name',
-                  'email': 'email' }
-    ticket_info = { 'service': valid_service }
+    user_info = {'username': 'ellen',
+                 'password': 'mamas&papas',
+                 'email': 'ellen@example.com'}
+    user_attr = {'givenName': 'first_name',
+                 'sn': 'last_name',
+                 'email': 'email'}
+    ticket_info = {'service': valid_service}
     valid_pgt_url = 'https://www.example.com/'
     invalid_pgt_url = 'http://www.example.com/'
+    invalid_attr = {'invalid':
+            ''.join(random.choice(string.ascii_letters) for x in range(10))}
 
     def setUp(self):
         """
@@ -320,9 +337,12 @@ class ServiceValidateViewTests(TestCase):
         self.old_user_attributes = getattr(settings, 'MAMA_CAS_USER_ATTRIBUTES', {})
         if not self.old_user_attributes:
             settings.MAMA_CAS_USER_ATTRIBUTES = self.user_attr
+        settings.MAMA_CAS_USER_ATTRIBUTES.update(self.invalid_attr)
         self.old_profile_attributes = getattr(settings, 'MAMA_CAS_PROFILE_ATTRIBUTES', {})
         if not self.old_profile_attributes:
             settings.MAMA_CAS_PROFILE_ATTRIBUTES = {}
+        self.old_valid_services = getattr(settings, 'MAMA_CAS_VALID_SERVICES', ())
+        settings.MAMA_CAS_VALID_SERVICES = (self.valid_service,)
 
     def tearDown(self):
         """
@@ -330,6 +350,7 @@ class ServiceValidateViewTests(TestCase):
         """
         settings.MAMA_CAS_USER_ATTRIBUTES = self.old_user_attributes
         settings.MAMA_CAS_PROFILE_ATTRIBUTES = self.old_profile_attributes
+        settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
 
     def test_service_validate_view(self):
         """
@@ -421,6 +442,10 @@ class ServiceValidateViewTests(TestCase):
         NOTE: this test will fail unless ``valid_pgt_url`` is configured with
         a valid proxy callback URL.
         """
+        if self.valid_pgt_url == 'https://www.example.com/':
+            raise ImproperlyConfigured("Set valid_pgt_url to a valid HTTPS "
+                                       "URL to successfully run this test")
+
         query_str = "?service=%s&ticket=%s&pgtUrl=%s" % (self.valid_service,
                                                          self.st.ticket,
                                                          self.valid_pgt_url)
@@ -472,6 +497,7 @@ class ServiceValidateViewTests(TestCase):
 
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.get('Content-Type'), 'text/xml')
+        self.assertIsNotNone(elem)
         # Because attribute order is not guaranteed, we take a list of all
         # configured attributes, compare each attribute found to make sure
         # it's present and then remove it from the temporary list.
@@ -484,6 +510,25 @@ class ServiceValidateViewTests(TestCase):
             attr_names.remove(attribute.tag)
         self.assertEqual(len(attr_names), 0)
 
+    def test_service_validate_view_invalid_service_url(self):
+        """
+        When ``MAMA_CAS_VALID_SERVICES`` is defined in the settings file, a
+        service string should be checked against the list of valid services.
+        If it does not match, a service authentication failure should be
+        returned.
+        """
+        query_str = "?service=%s&ticket=%s" % (self.invalid_service,
+                                               self.st.ticket)
+        response = self.client.get(reverse('cas_service_validate') + query_str)
+        tree = ElementTree(fromstring(response.content))
+        elem = tree.find(XMLNS + 'authenticationFailure')
+
+        self.assertIsNotNone(elem)
+        self.assertEqual(elem.get('code'), 'INVALID_SERVICE')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.get('Content-Type'), 'text/xml')
+
+
 class ProxyValidateViewTests(TestCase):
     """
     Test the ``ProxyValidateView`` view.
@@ -493,15 +538,18 @@ class ProxyValidateViewTests(TestCase):
     valid_service = 'http://www.example.com/'
     valid_service2 = 'http://ww2.example.com/'
     invalid_service = 'http://www.example.org/'
-    user_info = { 'username': 'ellen',
-                  'password': 'mamas&papas',
-                  'email': 'ellen@example.com' }
-    user_attr = { 'givenName': 'first_name',
-                  'sn': 'last_name',
-                  'email': 'email' }
-    ticket_info = { 'service': valid_service }
+    user_info = {'username': 'ellen',
+                 'password': 'mamas&papas',
+                 'email': 'ellen@example.com'}
+    user_attr = {'givenName': 'first_name',
+                 'sn': 'last_name',
+                 'email': 'email'}
+    ticket_info = {'service': valid_service}
     valid_pgt_url = 'https://www.example.com/'
     invalid_pgt_url = 'http://www.example.com/'
+    invalid_attr = {'invalid':
+            ''.join(random.choice(string.ascii_letters) for x in range(10))}
+
 
     def setUp(self):
         """
@@ -524,9 +572,13 @@ class ProxyValidateViewTests(TestCase):
         self.old_user_attributes = getattr(settings, 'MAMA_CAS_USER_ATTRIBUTES', {})
         if not self.old_user_attributes:
             settings.MAMA_CAS_USER_ATTRIBUTES = self.user_attr
+        settings.MAMA_CAS_USER_ATTRIBUTES.update(self.invalid_attr)
         self.old_profile_attributes = getattr(settings, 'MAMA_CAS_PROFILE_ATTRIBUTES', {})
         if not self.old_profile_attributes:
             settings.MAMA_CAS_PROFILE_ATTRIBUTES = {}
+        self.old_valid_services = getattr(settings, 'MAMA_CAS_VALID_SERVICES', ())
+        settings.MAMA_CAS_VALID_SERVICES = (self.valid_service,
+                                            self.valid_service2)
 
     def tearDown(self):
         """
@@ -534,6 +586,7 @@ class ProxyValidateViewTests(TestCase):
         """
         settings.MAMA_CAS_USER_ATTRIBUTES = self.old_user_attributes
         settings.MAMA_CAS_PROFILE_ATTRIBUTES = self.old_profile_attributes
+        settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
 
     def test_proxy_validate_view(self):
         """
@@ -565,7 +618,7 @@ class ProxyValidateViewTests(TestCase):
         When called with an invalid service identifier, a ``GET`` request
         to the view should return a validation failure.
         """
-        query_str = "?service=%s&ticket=%s" % (self.invalid_service, self.st.ticket)
+        query_str = "?service=%s&ticket=%s" % (self.invalid_service, self.pt.ticket)
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
         tree = ElementTree(fromstring(response.content))
         elem = tree.find(XMLNS + 'authenticationFailure')
@@ -580,7 +633,7 @@ class ProxyValidateViewTests(TestCase):
         When called with an invalid ticket identifier, a ``GET`` request
         to the view should return a validation failure.
         """
-        query_str = "?service=%s&ticket=%s" % (self.valid_service, self.invalid_st_str)
+        query_str = "?service=%s&ticket=%s" % (self.valid_service, self.invalid_pt_str)
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
         tree = ElementTree(fromstring(response.content))
         elem = tree.find(XMLNS + 'authenticationFailure')
@@ -635,7 +688,7 @@ class ProxyValidateViewTests(TestCase):
         elem = tree.find(XMLNS + 'authenticationSuccess/' + XMLNS + 'proxies')
         proxy = list(elem.getiterator(XMLNS + 'proxy'))
         self.assertEqual(len(proxy), 1)
-        self.assertEqual(proxy[0].text, self.valid_service)
+        self.assertEqual(proxy[0].text, 'http://www.example.com')
 
         # This request should fail as the ticket was consumed in the preceeding test
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
@@ -658,8 +711,8 @@ class ProxyValidateViewTests(TestCase):
                                                          validate=False,
                                                          user=self.user,
                                                          granted_by_pt=self.pt)
-        ticket_info2 = { 'service': self.valid_service2,
-                         'user': self.user }
+        ticket_info2 = {'service': self.valid_service2,
+                        'user': self.user}
         pt2 = ProxyTicket.objects.create_ticket(granted_by_pgt=pgt2,
                                                 **ticket_info2)
         query_str = "?service=%s&ticket=%s" % (self.valid_service2, pt2)
@@ -675,8 +728,8 @@ class ProxyValidateViewTests(TestCase):
         elem = tree.find(XMLNS + 'authenticationSuccess/' + XMLNS + 'proxies')
         proxy = list(elem.getiterator(XMLNS + 'proxy'))
         self.assertEqual(len(proxy), 2)
-        self.assertEqual(proxy[0].text, self.valid_service2)
-        self.assertEqual(proxy[1].text, self.valid_service)
+        self.assertEqual(proxy[0].text, 'http://ww2.example.com')
+        self.assertEqual(proxy[1].text, 'http://www.example.com')
 
     def test_proxy_validate_view_pgturl(self):
         """
@@ -687,8 +740,12 @@ class ProxyValidateViewTests(TestCase):
         NOTE: this test will fail unless ``valid_pgt_url`` is configured with
         a valid proxy callback URL.
         """
+        if self.valid_pgt_url == 'https://www.example.com/':
+            raise ImproperlyConfigured("Set valid_pgt_url to a valid HTTPS "
+                                       "URL to successfully run this test")
+
         query_str = "?service=%s&ticket=%s&pgtUrl=%s" % (self.valid_service,
-                                                         self.st.ticket,
+                                                         self.pt.ticket,
                                                          self.valid_pgt_url)
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
         tree = ElementTree(fromstring(response.content))
@@ -709,7 +766,7 @@ class ProxyValidateViewTests(TestCase):
         success with no ``ProxyGrantingTicket`` (``pgtUrl`` must be HTTPS).
         """
         query_str = "?service=%s&ticket=%s&pgtUrl=%s" % (self.valid_service,
-                                                         self.st.ticket,
+                                                         self.pt.ticket,
                                                          self.invalid_pgt_url)
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
         tree = ElementTree(fromstring(response.content))
@@ -731,13 +788,14 @@ class ProxyValidateViewTests(TestCase):
         """
         attr_names = settings.MAMA_CAS_USER_ATTRIBUTES.keys()
         attr_names.extend(settings.MAMA_CAS_PROFILE_ATTRIBUTES.keys())
-        query_str = "?service=%s&ticket=%s" % (self.valid_service, self.st.ticket)
-        response = self.client.get(reverse('cas_service_validate') + query_str)
+        query_str = "?service=%s&ticket=%s" % (self.valid_service, self.pt.ticket)
+        response = self.client.get(reverse('cas_proxy_validate') + query_str)
         tree = ElementTree(fromstring(response.content))
         elem = tree.find(XMLNS + 'authenticationSuccess/' + XMLNS + 'attributes')
 
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.get('Content-Type'), 'text/xml')
+        self.assertIsNotNone(elem)
         # Because attribute order is not guaranteed, we take a list of all
         # configured attributes, compare each attribute found to make sure
         # it's present and then remove it from the temporary list.
@@ -750,16 +808,36 @@ class ProxyValidateViewTests(TestCase):
             attr_names.remove(attribute.tag)
         self.assertEqual(len(attr_names), 0)
 
+    def test_proxy_validate_view_invalid_service_url(self):
+        """
+        When ``MAMA_CAS_VALID_SERVICES`` is defined in the settings file, a
+        service string should be checked against the list of valid services.
+        If it does not match, a proxy authentication failure should be
+        returned.
+        """
+        query_str = "?service=%s&ticket=%s" % (self.invalid_service,
+                                               self.pt.ticket)
+        response = self.client.get(reverse('cas_proxy_validate') + query_str)
+        tree = ElementTree(fromstring(response.content))
+        elem = tree.find(XMLNS + 'authenticationFailure')
+
+        self.assertIsNotNone(elem)
+        self.assertEqual(elem.get('code'), 'INVALID_SERVICE')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.get('Content-Type'), 'text/xml')
+
+
 class ProxyViewTests(TestCase):
     """
     Test the ``ProxyView`` view.
     """
     invalid_pgt_str = 'PGT-0000000000-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
     valid_service = 'http://www.example.com/'
-    user_info = { 'username': 'ellen',
-                  'password': 'mamas&papas',
-                  'email': 'ellen@example.com' }
-    ticket_info = { 'service': valid_service }
+    invalid_service = 'http://www.example.org/'
+    user_info = {'username': 'ellen',
+                 'password': 'mamas&papas',
+                 'email': 'ellen@example.com'}
+    ticket_info = {'service': valid_service}
 
     def setUp(self):
         """
@@ -773,6 +851,15 @@ class ProxyViewTests(TestCase):
                                                              user=self.user,
                                                              granted_by_st=self.st)
 
+        self.old_valid_services = getattr(settings, 'MAMA_CAS_VALID_SERVICES', ())
+        settings.MAMA_CAS_VALID_SERVICES = (self.valid_service,)
+
+    def tearDown(self):
+        """
+        Undo any modifications made to settings.
+        """
+        settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
+
     def test_proxy_view(self):
         """
         When called with no parameters, a ``GET`` request to the view should
@@ -844,3 +931,21 @@ class ProxyViewTests(TestCase):
         self.assertNotEqual(elem.text, 0)
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.get('Content-Type'), 'text/xml')
+
+    def test_proxy_view_invalid_service_url(self):
+        """
+        When ``MAMA_CAS_VALID_SERVICES`` is defined in the settings file, a
+        service string should be checked against the list of valid services.
+        If it does not match, a proxy authentication failure should be
+        returned.
+        """
+        query_str = "?targetService=%s&pgt=%s" % (self.invalid_service,
+                                                  self.pgt.ticket)
+        response = self.client.get(reverse('cas_proxy') + query_str)
+        tree = ElementTree(fromstring(response.content))
+        elem = tree.find(XMLNS + 'proxyFailure')
+
+        self.assertIsNotNone(elem)
+        self.assertEqual(elem.get('code'), 'INVALID_SERVICE')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.get('Content-Type'), 'text/xml')
